// Copyright (C) 2005-2007 Stefan Bauer <yoltia-maintain@freenet.de>

// This file is part of Yoltia.
//
// Yoltia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License Version 2
// as published by the Free Software Foundation.
//    
// Yoltia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Yoltia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#include "skurve.hpp"
#include <qpainter.h>
#include <qcursor.h>

SKurve::SKurve(QWidget* parent) : QWidget(parent , "kurve") {
  for (uchar zv = 1 ; zv > 0 ; zv++)
    data[zv] = zv;
  editMode = kein;
  setMinimumSize(256 , 256);
  setPaletteBackgroundColor("white");
  setPaletteForegroundColor("black");
  setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding , QSizePolicy::MinimumExpanding));
  setCursor(Qt::CrossCursor);
  setMouseTracking(true);
}

void SKurve::paintEvent(QPaintEvent*) {
  QPainter painter(this);
  painter.setPen(Qt::lightGray);
  for (uchar zv = 1 ; zv < 8 ; zv++) {
    painter.drawLine(qRound(zv * width() / 8.0) , 0 , qRound(zv * width() / 8.0) , height() - 1);
    painter.drawLine(0 , qRound(zv * height() / 8.0) - 1 , width() - 1 , qRound(zv * height() / 8.0) - 1);
  }
  painter.setPen(Qt::black);
  for (uchar zv = 1 ; zv > 0 ; zv++)
    painter.drawPoint(qRound(zv * (width() - 1.0) / 255.0) , qRound((height() - 1.0 - data[zv]) * (height() - 1.0) / 255.0));
}

void SKurve::mouseMoveEvent(QMouseEvent* maus) {
  uchar alt = qRound(maus->x() * 255.0 / (width() - 1.0)) , neu = qRound((height() - 1.0 - maus->y()) * 255.0 / (height() - 1.0));	//Mauspositionen mappen
  emit positionChanged(tr("Old: %1\nNew: %2").arg(alt).arg(neu));
  switch (editMode) {
    case frei :
      data[alt] = neu;
      update();
      break;
    case linie :
      {
	uchar n = alt > fixAlt ? fixNeu : neu , untGr = QMIN(alt , fixAlt);	//Vertikale Verschiebung der Geraden und untere Grenze bzw. horizontale Verschiebung der Geraden
	double m = (double) (neu - fixNeu) / (double) (alt - fixAlt);	//Anstieg der Geraden
	for (ushort zv = untGr ; zv <= QMAX(alt , fixAlt) ; zv++)	//Kein uchar, da sont Endlosschleife bei QMAX(alt , fixAlt) == 255
	  data[zv] = qRound(m * (zv - untGr) + n);
      }
      update();
    default :
      ;
  }
}

void SKurve::mousePressEvent(QMouseEvent* maus) {
  if (editMode == kein)
    switch (maus->button()) {
      case Qt::LeftButton :
	editMode = frei;
	break;
      case Qt::RightButton :
	fixAlt = qRound(QMAX(QMIN(maus->x() , width() - 1) , 0) * 256.0 / width());
	fixNeu = qRound((height() - 1.0 - QMAX(QMIN(maus->y() , height() - 1) , 0)) * 256.0 / height());
	editMode = linie;
      default :
	;
      }
    else {
      editMode = kein;
      emit dataChanged();
  }
}

void SKurve::leaveEvent(QEvent*) {
  emit positionChanged(0);
}
