// Copyright (C) 2005-2007 Stefan Bauer <yoltia-maintain@freenet.de>

// This file is part of Yoltia.
//
// Yoltia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License Version 2
// as published by the Free Software Foundation.
//    
// Yoltia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Yoltia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#include "slistedit.hpp"
#include <qlayout.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qsettings.h>

void dirCrawl(const QString dirPath , QStringList *list , const uint introCount) {
  QDir dir(dirPath , "*.[bB][mM][pP] *.[gG][iI][fF] *.[mM][nN][gG] *.[jJ][pP][gG] *.[jJ][pP][eE][gG] *.[pP][bB][mM] *.[pP][gG][mM] *.[pP][nN][gG] *.[pP][pP][mM] *.[xX][bB][mM] *.[xX][pP][mM]" , QDir::Name , QDir::Files);
  QString item;
  if (introCount == 0)
      for (uint zv = 0 ; zv < dir.count() ; zv++) {	//Adding all files to list
        item = dir.path() + "/" + dir[zv];
        list->append(item);
      }
    else
      for (uint zv = 0 ; zv < QMIN(dir.count() , introCount) ; zv++) {	//Adding intro files to list
        item = dir.path() + "/" + dir[zv];
        list->append(item);
      }
  dir.setNameFilter(QString::null);
  dir.setFilter(QDir::Dirs);
  for (uint zv = 2 ; zv < dir.count() ; zv++) {	//Recursive Call
    item = dir.path() + "/" + dir[zv];
    dirCrawl(item , list , introCount);
  }
}

SListEdit::SListEdit(QWidget *parent , QStringList list) : QDialog(parent , "listEdit" , true) {
  QGridLayout *container = new QGridLayout(this , 8 , 2 , 10 , -1 , "container");
  listView = new QListBox(this , "listView");
  info = new QLabel(this , "info");
  info->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
  container->addMultiCellWidget(listView , 0 , 7 , 0 , 0);
  container->addWidget(info , 0 , 1);
  connect(listView , SIGNAL(highlighted(int)) , this , SLOT(refreshInfo(int)));
  if (!list.isEmpty()) {
    listView->insertStringList(list);
    listView->setCurrentItem(0);
  }
  QPushButton *up = new QPushButton(tr("&Up") , this , "up");
  QPushButton *down = new QPushButton(tr("&Down") , this , "down");
  QPushButton *addF = new QPushButton(tr("&Add") , this , "addF");
  QPushButton *addD = new QPushButton(tr("Add di&rectory") , this , "addD");
  QPushButton *addIntro = new QPushButton(tr("Add &intro") , this , "addIntro");
  QPushButton *del = new QPushButton(tr("&Remove") , this , "del");
  QPushButton *ok = new QPushButton(tr("&Finished") , this , "ok");
  ok->setDefault(true);
  container->addWidget(up , 1 , 1);
  container->addWidget(down , 2 , 1);
  container->addWidget(addF , 3 , 1);
  container->addWidget(addD , 4 , 1);
  container->addWidget(addIntro , 5 , 1);
  container->addWidget(del , 6 , 1);
  container->addWidget(ok , 7 , 1);
  connect(up , SIGNAL(clicked()) , this , SLOT(pushUp()));
  connect(down , SIGNAL(clicked()) , this , SLOT(pushDown()));
  connect(del , SIGNAL(clicked()) , this , SLOT(pushAway()));
  connect(addF , SIGNAL(clicked()) , this , SLOT(addFiles()));
  connect(addD , SIGNAL(clicked()) , this , SLOT(addDir()));
  connect(addIntro , SIGNAL(clicked()) , this , SLOT(addIntro()));
  connect(ok , SIGNAL(clicked()) , this , SLOT(accept()));
  setCaption(tr("Browsing-List"));
}

void SListEdit::refreshInfo(int index) {
  info->setText(QString("%1 / %2").arg(index + 1).arg(listView->count()));
}

void SListEdit::pushUp() {
  const uint pos = listView->currentItem();
  if (pos) {
    QString temp = listView->text(pos);
    listView->changeItem(listView->text(pos - 1) , pos);
    listView->changeItem(temp , pos - 1);
    listView->setSelected(pos - 1 , true);
  }
}

void SListEdit::pushDown() {
  const uint pos = listView->currentItem();
  if (pos < listView->count() - 1) {
    QString temp = listView->text(pos);
    listView->changeItem(listView->text(pos + 1) , pos);
    listView->changeItem(temp , pos + 1);
    listView->setSelected(pos + 1 , true);
  }
}

void SListEdit::pushAway() {
  if (listView->count() > 1) {
    const uint pos = listView->currentItem();
    listView->removeItem(pos);
    listView->setSelected(QMIN(pos , listView->count() - 1) , true);
  }
}

void SListEdit::addFiles() {
  QStringList list = QFileDialog::getOpenFileNames(tr("All image files (*.[bB][mM][pP] *.[gG][iI][fF] *.[mM][nN][gG] *.[jJ][pP][gG] *.[jJ][pP][eE][gG] *.[pP][bB][mM] *.[pP][gG][mM] *.[pP][nN][gG] *.[pP][pP][mM] *.[xX][bB][mM] *.[xX][pP][mM]);;All files (*);;BMP (*.[bB][mM][pP]);;GIF (*.[gG][iI][fF]);;MNG (*.[mM][nN][gG]);;JPEG (*.[jJ][pP][gG] *.[jJ][pP][eE][gG]);;PNG (*.[pP][nN][gG]);;PNM (*.[pP][bB][mM] *.[pP][gG][mM] *.[pP][pP][mM]);;XBM (*.[xX][bB][mM]);;XPM (*.[xX][pP][mM])") , QSettings().readEntry("/Yoltia/history/dir" , QDir::homeDirPath()) , this , "openFilesDlg" , tr("Choose files")); 
  if (list.count())	//Files selected (not mandatory)
    listView->insertStringList(list , listView->currentItem() + 1);
  refreshInfo(listView->currentItem());
}

void SListEdit::addDir() {
  QStringList list;
  QString dir = QFileDialog::getExistingDirectory(QSettings().readEntry("/Yoltia/history/dir" , QDir::homeDirPath()) , this , "openDirDlg" , tr("Choose directory") , false);
  if (!dir.isEmpty()) {
    dirCrawl(dir , &list , 0);
    if (list.count())	//Files selected (not mandatory)
      listView->insertStringList(list , listView->currentItem() + 1);
    refreshInfo(listView->currentItem());
  }
}

void SListEdit::addIntro() {
  QStringList list;
  QString dir = QFileDialog::getExistingDirectory(QSettings().readEntry("/Yoltia/history/dir" , QDir::homeDirPath()) , this , "openDirDlg" , tr("Choose directory") , false);
  if (!dir.isEmpty()) {
    dirCrawl(dir , &list , QSettings().readNumEntry("/Yoltia/introN" , 2));
    if (list.count())	//Files selected (not mandatory)
      listView->insertStringList(list , listView->currentItem() + 1);
    refreshInfo(listView->currentItem());
  }
}

QStringList SListEdit::getList() {
  QStringList filelist;
  for (int zv = listView->count() - 1 ; zv >= 0 ; zv--)
    filelist.prepend(listView->text(zv));
  return filelist;
}
